﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace ConsoleApplication3
{
    public class Service<TDto, TDomainEntity>
    {
        Assembler<TDto, TDomainEntity> _assembler;
        IRepository<TDomainEntity> _repository;

        public Service(Assembler<TDto, TDomainEntity> assembler,
            IRepository<TDomainEntity> repository)
        {
            _assembler = assembler;
            _repository = repository;
        }

        public Assembler<TDto, TDomainEntity> Assembler { get { return _assembler; } }
        public IRepository<TDomainEntity> Repository { get { return _repository; } }

        public virtual List<TDto> Find(Func<TDomainEntity, bool> filter)
        {
            return _assembler.DomainEntitiesToDtos(_repository.Find(filter));
        }

        public virtual TDto GetById(long Id)
        {
            return _assembler.DomainEntityToDto(_repository.GetById(Id));
        }

        public virtual void Add(TDto dto)
        {
            _repository.Add(_assembler.DtoToDomainEntity(dto));
        }

        public virtual void Update(TDto dto)
        {
            _repository.Update(_assembler.DtoToDomainEntity(dto));
        }

        public virtual void Delete(TDto dto)
        {
            _repository.Delete(_assembler.DtoToDomainEntity(dto));
        }
    }

    public interface IRepository<TDomainEntity>
    {
        IEnumerable<TDomainEntity> Find(Func<TDomainEntity, bool> filter);
        TDomainEntity GetById(long Id);
        void Add(TDomainEntity domainEntity);
        void Delete(TDomainEntity domainEntity);
        void Update(TDomainEntity domainEntity);
    }

    public abstract class Assembler<TDto, TDomainEntity>
    {
        public abstract TDomainEntity DtoToDomainEntity(TDto dto);
        public abstract TDto DomainEntityToDto(TDomainEntity domainEntity);

        public List<TDto> DomainEntitiesToDtos(IEnumerable<TDomainEntity> domainEntityList)
        {
            List<TDto> dtos = Activator.CreateInstance<List<TDto>>();
            foreach (TDomainEntity domainEntity in domainEntityList)
            {
                dtos.Add(DomainEntityToDto(domainEntity));
            }
            return dtos;
        }

        public List<TDomainEntity> DtosToDomainEntities(IEnumerable<TDto> dtoList)
        {
            List<TDomainEntity> domainEntities = Activator.CreateInstance<List<TDomainEntity>>();
            foreach (TDto dto in dtoList)
            {
                domainEntities.Add(DtoToDomainEntity(dto));
            }
            return domainEntities;
        }
    }
}
