﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace ConsoleApplication3
{
    public class AlbumService : Service<AlbumDto, Album>
    {
        public AlbumService()
            : base(new AlbumAssembler(), new AlbumRepository()) { }


        public AlbumDto FindByAlbumTitle(string title)
        {
            return Find(a => a.Title == title).First<AlbumDto>();
        }

        public override void Add(AlbumDto dto)
        {
            base.Add(dto);
        }
    }

    public class AlbumRepository : IRepository<Album>
    {
       
        #region IRepository<Album> Members
        List<Album> _albums = new List<Album>
        {
            new Album {Id=1, Title="Rock Out", Artist=new Artist{Name="Rock Dudes"}},
            new Album {Id=2, Title="Rock On", Artist=new Artist{Name="Rock Dudes"}},
            new Album {Id=3, Title="Yee haw", Artist=new Artist{Name="The Hicks"}}
        };

        public IEnumerable<Album> Find(Func<Album, bool> filter)
        {
            return _albums.Where(filter);
        }

        public Album GetById(long Id)
        {
            return _albums.Find(a => a.Id == Id);
        }

        public void Add(Album domainEntity)
        {
            _albums.Add(domainEntity);
        }

        public void Delete(Album domainEntity)
        {
            _albums.Remove(domainEntity);
        }

        public void Update(Album domainEntity)
        {
            Album album = _albums.Find(a => a.Id == domainEntity.Id);
            _albums.Remove(album);
            _albums.Add(domainEntity);
        }
        #endregion
    }

    public class AlbumAssembler : Assembler<AlbumDto, Album>
    {
        public override AlbumDto DomainEntityToDto(Album domainEntity)
        {
            return new AlbumDto
            {
                Id = domainEntity.Id,
                ArtistName = domainEntity.Artist.Name,
                Title = domainEntity.Title
            };
        }

        public override Album DtoToDomainEntity(AlbumDto dto)
        {
            Album album = new Album { Id = dto.Id, Title = dto.Title };
            album.Artist = new Artist { Name = dto.ArtistName };

            return album;
        }
    }

    public class AlbumDto
    {
        public long Id { get; set; }
        public string ArtistName { get; set; }
        public string Title { get; set; }
    }

    public class Artist
    {
        public string Name { get; set; }
    }

    public class Album
    {
        public long Id { get; set; }
        public string Title { get; set; }
        public Artist Artist { get; set; }
    }

}
